function [rat counter_c] = cep(Nr,N,vec_mu,ex,flag_KSF,flag_plot)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%input
% Nr             = no. of runs
% N              = no. of measurements
% vec_mu         = grid for \mu
% ex             = which example
% flag_KSF       = 0: KSF criterion;
%                  1: KSFM criterion;
%                  2: KSFM criterion (the search for the optimum threshold
%                     is not performed on the \ell_grid, 
%                     but for k\in\\{1,\ldots,M-1\}.
% flag_plot     = 2 : plots L versus \mu_{KSF} and k versus \mu_{KSF}
%               = 3 : plots the true and the estimated spec.
%output
% rat
%       rat(1)  = unused in the current version
%       rat(2)  = \rho_{genie}
%       rat(3)  = \rho_{KSF}
%       rat(4)  = \rho_{BIC}
%       rat(5)  = \rho_{UMPUT}
% counter_c     = average no. of coeff. retained by each method
%                  counter_c(1) - unused in the current version   
%                  counter_c(2) - genie
%                  counter_c(3) - KSF
%                  counter_c(4) - BIC
%                  counter_c(5) - UMPUT
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%INIT
M = N/2+1;
% vectors for total variance
TV = zeros(Nr,1);
TV_KSF = zeros(Nr,1);
TV_BIC = zeros(Nr,1);
TV_UMPUT = zeros(Nr,1);
TV_genie = zeros(Nr,length(vec_mu));

mat_counter_c_genie = zeros(Nr,length(vec_mu));
%counter for how many coeff. have been retained
counter_c = zeros(1,3);

%ARMA models
if ex==1,
% broadband MA with a SMALL dynamic range of the log-spectrum 
% Stoica-Sandgren Model I
    A = 1;
    B = [1 0.55 0.15]; 
elseif ex==2,
% broadband MA with a MEDIUM dynamic range of the log-spectrum 
% Stoica-Sandgren Model II
    A = 1;
    B = [1 0.4574 0.2157 0.3951 0.1383];
elseif ex==3,    
% narrowband ARMA with a LARGE dynamic range of the log-spectrum 
% Stoica-Sandgren Model III
    A = [1 -1.55 0.95];
    B = [1 0.75 0.35];
elseif ex==4,
% broadband AR with a SMALL dynamic range of the log-spectrum     
% Lai et al. Model III    
     A = [1 -1.5 0.7 -0.1];
     B = 1;
elseif ex==5,
% broadband MA with a MEDIUM dynamic range of the log-spectrum     
% Lai et al. Model IV 
     A = 1;
     B = [1 -0.3 -0.6 -0.3 0.6];
else
    fprintf('ERROR: incorrect value for the param. ex.\n');
    return;
end

%compute code length for the structure
Lk = comp_Lk(M);

%compute true cepstral coeff. 
%c_p <-> c_j
[c_p,Phi_p,flag_err] = comp_c_p(A,B,N);
if flag_err==1,
    return;
end


if N==2048,
    %Plot the true spectrum
    plot_true_spec(Phi_p,N,ex);
    %Plot the true cepstral coeff.
    plot_c_p(c_p,N,ex); 
end

%for each run
for rr=1:Nr
    %generate data
    y = gen_data(A,B,N);
    
    %estimate the cepstral coeff.
    %c_e <-> \hat c_j
    Phi_p_tmp = (abs(fft(y)).^2)./N;
    c_e = ifft(log(Phi_p_tmp));
    c_e(1) = c_e(1) + 0.57721; % Euler-Mascheroni constant
    if max(abs( c_e(2:N/2)-c_e(N:-1:N/2+2) ))>10^(-10),
        fprintf('ERROR in the estimated cepstral coeff.!\n');
        return;
    else
       c_e(N:-1:N/2+2) = c_e(2:N/2); 
    end
        
    %compute thresholds
    %KSF
    if flag_KSF==0,
        mu_KSF = comp_mu_KSF(c_e,N,M,Lk,vec_mu,ex,flag_plot);
    elseif  flag_KSF==1,
        mu_KSF = comp_mu_KSF_mod(c_e,N,M,Lk,vec_mu,ex,flag_plot);
    elseif  flag_KSF==2,
        mu_KSF = comp_mu_KSF_mod_2(c_e,N,M,Lk,vec_mu);
    else
    end        
    %BIC
    mu_BIC = 1 + sqrt(log(M));
    %UMPUT
    if ex<=3,
        II = ex;
    elseif ex==4,
        II = 1;
    elseif ex==5,
        II = 2;
    else
        fprintf('ERROR: incorrect value for the param. ex.\n');
        return;
    end     
    mu_UMPUT = (5-II) + (N-128)/1920;
    
    %compute c_til
    % \check c <-> c_til
    c_til_KSF = comp_c_til(c_e,N,mu_KSF);
    c_til_BIC = comp_c_til(c_e,N,mu_BIC);
    c_til_UMPUT = comp_c_til(c_e,N,mu_UMPUT);
    
    %plot the estimated spectrum and the true spectrum
    if flag_plot==3,
        H = figure;
        plot_est_spec(Phi_p,c_til_KSF,Phi_p_tmp,N,H,1,ex);
        plot_est_spec(Phi_p,c_til_BIC,Phi_p_tmp,N,H,2,ex);
        plot_est_spec(Phi_p,c_til_UMPUT,Phi_p_tmp,N,H,3,ex);
        pause
        close(H);
    end
            
    %how many coeff. have been retained by KSF, BIC, UMPUT
    counter_c = counter_c + [
        length(find(c_til_KSF~=0)) 
        length(find(c_til_BIC~=0)) 
        length(find(c_til_UMPUT~=0))]';
        
    %compute TV
    TV(rr) = sum((c_p-c_e).^2);
    TV_KSF(rr) = sum((c_p-c_til_KSF).^2);
    TV_BIC(rr) = sum((c_p-c_til_BIC).^2);
    TV_UMPUT(rr) = sum((c_p-c_til_UMPUT).^2);
    [TV_genie(rr,:) mat_counter_c_genie(rr,:)] = comp_TV_genie(c_p,c_e,N,vec_mu);
       
end %rr


counter_c_genie1 = 0;
%genie (2): choose \mu such that the average TV is minimized
[dum,ind] = min(mean(TV_genie,1));
counter_c_genie2 = mean(mat_counter_c_genie(:,ind));

rat = mean(TV)./[mean(min(TV_genie,[],2)) min(mean(TV_genie,1)) mean(TV_KSF) mean(TV_BIC) mean(TV_UMPUT)];

%average no. of coeff. that have been retained
counter_c = [counter_c_genie1 counter_c_genie2 (counter_c/Nr)];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Generate data
function  y = gen_data(A,B,N)

%input:
%   A = AR part of the model
%   B = MA part of the model
%   N = no. of measurements
%output:
%   y = simulated data

x = randn(N,1);
y = filter(B,A,x);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Compute TV genie
function [vec_TV_genie counter_c_genie] = comp_TV_genie(c_p,c_e,N,vec_mu)

%input:
%   c_p              = true cepstral coeff.
%   c_e              = estimated cepstral coeff.
%   N                = no. of measurements
%   vec_mu           = grid for the threshold \mu
%output:
%   vec_TV_genie    = TV genie for each value of \mu
%   counter_c_genie = no. of the retained coeff. for each value of \mu

lm = length(vec_mu);
vec_TV_genie = zeros(1,lm);
counter_c_genie = zeros(1,lm);

for i=1:lm,
    mu = vec_mu(i);
    c_til = comp_c_til(c_e,N,mu);
    vec_TV_genie(i) = sum((c_p-c_til).^2);
    counter_c_genie(i) = length(find(c_til~=0));
end
    







